@@ -98,14 +98,31 @@ prepare_main_window (GisDriver *driver)
   GisDriverPrivate *priv = gis_driver_get_instance_private (driver);
   GdkGeometry size_hints;
 
-  size_hints.min_width = 1024;
-  size_hints.min_height = 768;
-  size_hints.win_gravity = GDK_GRAVITY_CENTER;
+  if (gis_driver_is_small_screen (driver))
+    {
+      GtkWidget *child, *sw;
 
-  gtk_window_set_geometry_hints (priv->main_window,
-                                 NULL,
-                                 &size_hints,
-                                 GDK_HINT_MIN_SIZE | GDK_HINT_WIN_GRAVITY);
+      child = g_object_ref (gtk_bin_get_child (GTK_BIN (priv->main_window)));
+      gtk_container_remove (GTK_CONTAINER (priv->main_window), child);
+      sw = gtk_scrolled_window_new (NULL, NULL);
+      gtk_widget_show (sw);
+      gtk_container_add (GTK_CONTAINER (priv->main_window), sw);
+      gtk_container_add (GTK_CONTAINER (sw), child);
+      g_object_unref (child);
+
+      gtk_window_maximize (priv->main_window);
+    }
+  else
+    {
+      size_hints.min_width = 1024;
+      size_hints.min_height = 768;
+      size_hints.win_gravity = GDK_GRAVITY_CENTER;
+
+      gtk_window_set_geometry_hints (priv->main_window,
+                                     NULL,
+                                     &size_hints,
+                                     GDK_HINT_MIN_SIZE | GDK_HINT_WIN_GRAVITY);
+    }
 
   gtk_window_set_titlebar (priv->main_window,
                            gis_assistant_get_titlebar (priv->assistant));
@@ -203,6 +220,19 @@ gis_driver_get_mode (GisDriver *driver)
   return priv->mode;
 }
 
+gboolean
+gis_driver_is_small_screen (GisDriver *driver)
+{
+  GdkScreen *screen;
+
+  screen = gdk_screen_get_default ();
+
+  if (g_getenv ("GIS_SMALL_SCREEN"))
+    return TRUE;
+
+  return gdk_screen_get_height (screen) < 800;
+}
+
 static void
 gis_driver_get_property (GObject      *object,
                          guint         prop_id,
