@@ -45,7 +45,7 @@
 #define INPUT_SOURCE_TYPE_XKB "xkb"
 #define INPUT_SOURCE_TYPE_IBUS "ibus"
 
-#define MIN_ROWS 6
+#define MIN_ROWS 5
 
 struct _CcInputChooserPrivate
 {
@@ -274,6 +274,9 @@ sync_checkmark (GtkWidget *row,
 	else
 	        should_be_visible = g_strcmp0 (widget->id, priv->id) == 0 && g_strcmp0 (widget->type, priv->type) == 0;
         gtk_widget_set_opacity (widget->checkmark, should_be_visible ? 1.0 : 0.0);
+
+        if (widget->is_extra && should_be_visible)
+                widget->is_extra = FALSE;
 }
 
 static void
@@ -283,6 +286,7 @@ sync_all_checkmarks (CcInputChooser *chooser)
 
         gtk_container_foreach (GTK_CONTAINER (priv->input_list),
                                sync_checkmark, chooser);
+        gtk_list_box_invalidate_filter (GTK_LIST_BOX (priv->input_list));
 }
 
 static GtkWidget *
@@ -319,6 +323,36 @@ no_results_widget_new (void)
         return widget;
 }
 
+static void
+choose_non_extras_foreach (GtkWidget *row,
+                           gpointer   user_data)
+{
+        GtkWidget *child;
+        InputWidget *widget;
+        guint *count = user_data;
+
+        *count += 1;
+        if (*count > MIN_ROWS)
+                return;
+
+        child = gtk_bin_get_child (GTK_BIN (row));
+        widget = get_input_widget (child);
+        if (widget == NULL)
+                return;
+
+        widget->is_extra = FALSE;
+}
+
+static void
+choose_non_extras (CcInputChooser *chooser)
+{
+        CcInputChooserPrivate *priv = cc_input_chooser_get_instance_private (chooser);
+        guint count = 0;
+
+        gtk_container_foreach (GTK_CONTAINER (priv->input_list),
+                               choose_non_extras_foreach, &count);
+}
+
 static void
 add_rows_to_list (CcInputChooser  *chooser,
 	          GList            *list,
@@ -344,11 +378,7 @@ add_rows_to_list (CcInputChooser  *chooser,
 		}
 		g_hash_table_add (priv->inputs, key);
 
-		if (g_hash_table_size (priv->inputs) > MIN_ROWS)
-			is_extra = TRUE;
-		else
-			is_extra = FALSE;
-		widget = input_widget_new (chooser, type, id, is_extra);
+		widget = input_widget_new (chooser, type, id, TRUE);
 		gtk_container_add (GTK_CONTAINER (priv->input_list), widget);
 	}
 }
@@ -390,6 +420,8 @@ get_locale_infos (CcInputChooser *chooser)
 	add_rows_to_list (chooser, list, INPUT_SOURCE_TYPE_XKB, id);
 	g_list_free (list);
 
+        choose_non_extras (chooser);
+
 	list = gnome_xkb_info_get_all_layouts (priv->xkb_info);
 	add_rows_to_list (chooser, list, INPUT_SOURCE_TYPE_XKB, id);
 	g_list_free (list);
