@@ -13,8 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by: Matthias Clasen <mclasen@redhat.com>
  */
@@ -231,8 +230,8 @@ add_other_users_language (GHashTable *ht)
 }
 
 static void
-insert_language (GHashTable *ht,
-                 const char *lang)
+insert_language_internal (GHashTable *ht,
+                          const char *lang)
 {
         gboolean has_translations;
         char *label_own_lang;
@@ -252,7 +251,7 @@ insert_language (GHashTable *ht,
 
         g_debug ("We have translations for %s", lang);
 
-        key = g_strdup_printf ("%s.utf8", lang);
+        key = g_strdup (lang);
 
         label_own_lang = gnome_get_language_from_locale (key, key);
         label_current_lang = gnome_get_language_from_locale (key, NULL);
@@ -274,6 +273,16 @@ insert_language (GHashTable *ht,
         g_free (label_untranslated);
 }
 
+static void
+insert_language (GHashTable *ht,
+                 const char *lang)
+{
+        char *utf8_variant = g_strconcat (lang, ".utf8", NULL);
+        insert_language_internal (ht, lang);
+        insert_language_internal (ht, utf8_variant);
+        g_free (utf8_variant);
+}
+
 static void
 insert_user_languages (GHashTable *ht)
 {
@@ -301,7 +310,14 @@ cc_common_language_get_initial_languages (void)
         ht = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
         insert_language (ht, "en_US");
+#if 0
+        /* Having 9 languages in the list initially makes the window
+         * too high. With 8 languages, we end up exactly 768 pixels
+         * high. Sadly, that means we can't affort to show English
+         * twice.
+         */
         insert_language (ht, "en_GB");
+#endif
         insert_language (ht, "de_DE");
         insert_language (ht, "fr_FR");
         insert_language (ht, "es_ES");
